"use strict";

// Handle squirrel events
// cf. https://github.com/electron/windows-installer#handling-squirrel-events
if (require("electron-squirrel-startup")){
  return;
}

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;

const langdetect = require("langdetect");
const fetch = require("node-fetch");
const path = require("path");
const fs = require("fs");
const electron = require("electron");
// Various electron modules we'll need
const {app, ipcMain, autoUpdater, BrowserWindow } = electron;
// Detect dev environment
const dev = require("electron-is-dev");
// Detect debug mode
const debug = dev || !!process.env.DEBUG;

// Set Application User Model ID
// See http://electron.atom.io/docs/api/auto-updater/#windows
// and https://github.com/electron/electron/issues/4866
app.setAppUserModelId("com.squirrel.fleex.Fleex");

// A global object to cache stuff
global.cache = {
  netflixSubs: []
};

// A global reference to the path to our `app` folder
global.appPath = path.resolve(dev ? process.cwd() : process.resourcesPath, "app");

// The app version
const appVersion = require(path.join(global.appPath, "package.json")).version;

// Add widevine for Netflix
if(process.platform == "darwin"){
  app.commandLine.appendSwitch("widevine-cdm-path", path.join(global.appPath, `plugins/darwin/widevinecdmadapter.plugin`));
} else if(process.platform == "win32"){
  app.commandLine.appendSwitch("widevine-cdm-path", path.join(global.appPath, `plugins/win32/${process.arch}/widevinecdmadapter.dll`));
}
app.commandLine.appendSwitch("widevine-cdm-version", "1.4.8.866");

// Bugfix with Webchimera on Windows
// See https://github.com/RSATom/WebChimera.js/wiki/Electron-v0.36.x-compatibility-issue-on-Windows
if(process.platform == "win32"){
  process.env["VLC_PLUGIN_PATH"] = path.join(global.appPath, "node_modules/wcjs-prebuilt/bin/plugins");
}
// Bugfix with Webchimera on OSX, caused by Electron builder de-referencing symlinks
// See https://github.com/electron-userland/electron-packager/issues/362#issuecomment-222986689
else if(process.platform == "darwin"){
  var libPath = path.join(global.appPath, "node_modules/wcjs-prebuilt/bin/lib");
  var vlcLibPath = path.resolve(libPath, "vlc/lib");
  var libvlccore = "libvlccore.8.dylib";
  fs.lstat(path.resolve(vlcLibPath, libvlccore), function(err, stats){
    if(!stats.isSymbolicLink()){
      fs.unlink(path.resolve(vlcLibPath, libvlccore), function(){
        fs.symlink(path.resolve(libPath, libvlccore), path.resolve(vlcLibPath, libvlccore), function(){});
      });
    }
  })
}

// In dev mode, use dev server with automatic refresh and hot 
// module replacement.
if(dev){
  const webpack = require("webpack");
  const WebpackDevServer = require("webpack-dev-server");
  const config = require("../webpack.config.dev.js");
  const compiler = webpack(config);
  const server = new WebpackDevServer(compiler, {
    hot: true,
    contentBase: config.devServer.contentBase,
    publicPath: config.devServer.publicPath,
    stats: { colors: true },
    webPreferences: {
      plugins: true
    }
  });
  server.listen(1338);
}
// In production mode, use express server
else {
  const express = require("express");
  const server = express();
  server.use(express.static(global.appPath));
  server.listen(1338);
}

function createWindow () {
  // Create the browser window, with a 16/9 aspect ratio
  mainWindow = new BrowserWindow({
      width: 1067, 
      height: 600,
      titleBarStyle: "hidden",
      backgroundColor: "#1D2626"
  });
  
  // Expose Cadmium in Netflix webviews
  const cadmiumFilter = {urls: ["*://*/*/cadmium.js", "*://*/*/bvui.js", "*://*/*/bk", "*://*/*/*bootstrap.js"]};
  mainWindow.webContents.session.webRequest.onBeforeRequest(cadmiumFilter, function(details, callback){
      // Add trailing "&" to avoid being caught in endless loop
      const redirectURL = `https://fleex.tv/Api/Netflix/ScriptWithInternalsExposed?originalScriptUrl=${details.url}&`;
      callback({redirectURL});
  });
  
  // Intercept subtitles requests in Netflix webviews
  const subsFilter = { urls: ["*://*.nflximg.com/*", "*://*.nflxvideo.net/*"]}
  mainWindow.webContents.session.webRequest.onBeforeSendHeaders(subsFilter, function(details, callback){
    // We're looking for a time text request made by an actual tab
    const isTimedTextRequest = 
      (details.url.indexOf("nflximg.com")!==-1 && details.url.indexOf("dfxp")!==-1) ||
      (details.url.indexOf("nflxvideo.net")!==-1 && details.url.indexOf("random")==-1 && details.url.indexOf("/range/")==-1);
    if(isTimedTextRequest){
      // Download the subtitles
      fetch(details.url)
        .then(response => response.ok ? response.text() : Promise.reject(response.statusText))
        .then(subText => {
          if(subText.indexOf("http://www.w3.org/ns/ttml")!==-1){
            const strippedSubText = subText.replace(/<[^>]*>/g, "");
            const lang = langdetect.detectOne(strippedSubText) || "un";
            global.cache.netflixSubs.push({
              url: details.url,
              language: lang,
              content: subText
            });
          }
        })
        .catch(function(){ /* Do nothing */ })
    }
    callback({cancel: false});
  });

  // In debug mode, open devtools
  debug && mainWindow.webContents.openDevTools();

  // Load the electron-app.html page
  mainWindow.loadURL("http://localhost:1338/electron-app.html");

  // Emitted when the window is closed.
  mainWindow.on("closed", function() {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null;
  });
}

// Setup the auto updater
autoUpdater.on("update-available", (event) => {
  mainWindow.webContents.send("update-available")
}).on("update-downloaded", (event, releaseNotes, releaseName, releaseDate, updateURL) => {
  mainWindow.webContents.send("update-downloaded", releaseNotes, releaseName, releaseDate, updateURL);
}).on("error", (event, error) => {
  mainWindow && mainWindow.webContents && mainWindow.webContents.send("update-error", error);
}).on("checking-for-update", (event) => {
  mainWindow.webContents.send("checking-for-update");
}).on("update-not-available", (event) => {
  mainWindow.webContents.send("update-not-available");
});
const autoUpdaterFeedUrl = process.platform === "darwin" ?
  `https://fleex.tv/Api/ElectronApp/OsxUpdateFeed?version=${appVersion}`:
  process.arch === "x64" ? "https://download.fleex.tv/electron-app/win":
  "https://download.fleex.tv/electron-app/win-ia32";
autoUpdater.setFeedURL(autoUpdaterFeedUrl);

ipcMain.on("check-for-updates", (event, arg) => {
  autoUpdater.checkForUpdates();
});

ipcMain.on("quit-and-install-update", (event, arg) => {
  autoUpdater.quitAndInstall();
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
app.on("ready", createWindow);

// Quit when all windows are closed.
app.on("window-all-closed", function () {
    // On OS X it is common for applications and their menu bar
    // to stay active until the user quits explicitly with Cmd + Q
    if (process.platform !== "darwin") {
        app.quit();
    }
});

app.on("activate", function () {
    // On OS X it"s common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (mainWindow === null) {
        createWindow();
    }
});